@@ -1,10 +1,34 @@
+#' dataset of shanghai composite index
+#'
+#' The daily historical Shanghai Composite Index from the beginning of the Index to the end of July 2020
+#'
+#' @format A data frame with 7240 rows and 15 variables:
+#' \describe{
+#'   \item{symbol}{stock ticker symbol}
+#'   \item{name}{stock ticker name}
+#'   \item{date}{trade date}
+#'   \item{open}{stock price at the open of trading}
+#'   \item{high}{stock price at the highest point during trading}
+#'   \item{low}{stock price at the lowest point during trading}
+#'   \item{close}{stock price at the close of trading}
+#'   \item{prev_close}{stock price at the close of previous trading day}
+#'   \item{change_pct}{change percentage of stock close price}
+#'   \item{volume}{number of shares traded}
+#'   \item{amount}{monetary value of shares traded}
+#'   \item{turnover}{rate of shares traded over total}
+#'   \item{cap_market}{tradable market capitalisation}
+#'   \item{cap_total}{total market capitalisation}
+#'   \item{unit}{price unit, such as in CNY/USD}
+#' }
+#' 
+"ssec"
 
 #' query stock market data 
 #' 
 #' \code{md_stock} provides an interface to query EOD (end of date) stock prices.
 #' 
 #' @param symbol symbols of stock shares.
-#' @param source the available data sources are 'yahoo' (\url{http://finance.yahoo.com}) and '163' (\url{http://money.163.com}).
+#' @param source the available data sources are 'yahoo' (\url{https://finance.yahoo.com}) and '163' (\url{https://money.163.com}).
 #' @param type the data type, including history, adjfactor and spot. Default is history.
 #' @param freq default is daily. It supports daily, weekly and monthly for yahoo data; daily for 163 data.
 #' @param date_range date range. Available value including '1m'-'11m', 'ytd', 'max' and '1y'-. Default is '3y'.
@@ -17,7 +41,7 @@
 #' 
 #' 
 #' @examples 
-#' \donttest{
+#' \dontrun{
 #' # Example I
 #' # query history prices from yahoo
 #' dt_yahoo1 = md_stock(symbol=c("^GSPC", "000001.SS"))
